#########
######
###
  Simple Add Layer
    (C) あかつきみさき(みくちぃP)

  このスクリプトについて
    コンポジション設定に基づいた新規レイヤーを作成します.
    レイヤーを選択している場合はその一つ上に作成します.

    追加可能な種類のレイヤー
      ・平面レイヤー
      ・調整レイヤー
      ・ヌルレイヤー
      ・ヌルレイヤー(3D)
      ・ライトレイヤー
      ・カメラレイヤー
      ・テキストレイヤー
      ・シェイプレイヤー

    Trim機能について
      新規レイヤーののスタートタイム・インポイント・アウトポイント・伸縮を,選択したレイヤーの値に一致させます.

  使用方法
    コンポジションをアクティブにしてからスクリプトを実行してください.

  動作環境
    Adobe After Effects CS5以上

  バージョン情報
    2018/10/11 Ver 1.1.0 Update
      調整レイヤー追加時,ラベルを通常追加時と同じ色にした.

    2014/08/07 Ver 1.0.0 Release
###
######
#########

SALData = ( ->
  scriptName          = "Simple Add Layer"
  scriptURLName       = "SimpleAddLayer"
  scriptVersionNumber = "1.1.0"
  scriptURLVersion    = 1.10
  canRunVersion       = 10.0
  guid                = "{544E36ED-D16A-485F-B887-8B273C79291E}"

  return{
    getScriptName         : -> scriptName
    ,
    getScriptURLName      : -> scriptURLName
    ,
    getScriptVersionNumber: -> scriptVersionNumber
    ,
    getScriptURLVersion   : -> scriptURLVersion
    ,
    getGuid               : -> guid
    ,
    getCanRunVersion      : -> canRunVersion
   }
)()

_STRINGS_JP =
  AUTHOR   : "あかつきみさき (SUNRISE MOON)"
  HELP     : ""

_STRINGS_EN =
  AUTHOR   : "Misaki_Akatsuki (SUNRISE MOON)"
  HELP     : ""

#色のクラス(16進数コード用)
class ColorClass
  constructor: (name, colorType) ->
    @name        = String name
    @colorType   = String colorType
    @colorCode   = name.split("#")[1]
    @colorCode16 = String "0x" + @colorCode
    @r           = @colorCode.slice(0, 2)
    @g           = @colorCode.slice(2, 4)
    @b           = @colorCode.slice(4, 6)
    @rgb256      = [parseInt(@r,16), parseInt(@g,16), parseInt(@b,16),1]
    @rgb         = [parseInt(@r,16)/255, parseInt(@g,16)/255, parseInt(@b,16)/255,1]
    @hsv         = rgb2hsv @rgb, true
    @h           = @hsv[0]
    @s           = @hsv[1]
    @v           = @hsv[2]
    @cmyk        = rgb2cmyk @rgb256
    @c           = @cmyk[0]
    @m           = @cmyk[1]
    @y           = @cmyk[2]
    @k           = @cmyk[3]

# 環境設定初期化
Settings::initSetting = (sectionName, keyName, value) ->
  unless app.settings.haveSetting(sectionName, keyName)
    app.settings.saveSetting sectionName, keyName, value

  return app.settings.getSetting(sectionName, keyName)


# CURRENT_COLOR = app.settings.initSetting(SALData.getScriptName(), "LAYER_COLOR", WHITE.toSource()).toSource()
# alert CURRENT_COLOR, app.settings.initSetting(SALData.getScriptName(), "LAYER_COLOR", WHITE.toSource())


AVLayer::addSolidSmart = (name) ->
  @containingComp.layers.addSolid(BLACK, "#{name}", @containingComp.width, @containingComp.height, @containingComp.pixelAspect, @containingComp.duration)
  return @

LayerCollection::addAdjustment = (color, name, width, height, pixelAspect, duration) ->
  addLayer = @addSolid(color, name, width, height, pixelAspect, duration)
  addLayer.adjustmentLayer = true
  return addLayer

AVLayer::addAdjustmentSmart = (name) ->
  @containingComp.layers.addAdjustment(BLACK, "#{name}", @containingComp.width, @containingComp.height, @containingComp.pixelAspect, @containingComp.duration)
  return @

# 指定したレイヤーの長さにマッチさせる
# 真偽返す
TextLayer::matchLength   =
LightLayer::matchLength  =
ShapeLayer::matchLength  =
CameraLayer::matchLength =
AVLayer::matchLength     = (layer) ->
  unless layer instanceof AVLayer or layer instanceof LightLayer or layer instanceof CameraLayer or layer instanceof ShapeLayer or layer instanceof TextLayer
    return false

  @startTime = layer.startTime
  @stretch   = layer.stretch
  @inPoint   = layer.inPoint
  @outPoint  = layer.outPoint
  return true


# 許容バージョンを渡し,実行できるか判別
runAEVersionCheck = (AEVersion) ->
  if parseFloat(app.version) < AEVersion
    alert "This script requires After Effects #{AEVersion.toFixed(1)} or greater."
    return false
  else
    return true

# コンポジションを選択しているか確認する関数
isCompActive = (selComp) ->
  unless(selComp and selComp instanceof CompItem)
    # alert "Select Composition"
    return false
  else
    return true

# レイヤーを選択しているか確認する関数
isLayerSelected = (selLayers) ->
  if selLayers.length is 0
    # alert "Select Layers"
    return false
  else
    return true

getLocalizedText = (str) ->
  (if app.language is Language.JAPANESE then str.jp else str.en)

buildUI = (thisObj) ->
  MAXIMUM_SIZE = 20

  myUI = (if (thisObj instanceof Panel) then thisObj else new Window("palette", SALData.getScriptName(), `undefined`,
    resizeable: true
  ))

  thisUI =
  """
    Group {
      alignment:["fill", "fill"],
      alignChildren:["fill", "top"],
      orientation:"column",
      spacing: 1,
      margins: [0, 0, 0, 0],

      oGrp1: Group {
        orientation:"row",
        spacing:0,
        margins:[0,0,0,0],
        alignment:["fill", "top"],

        layerTypeDdl: DropDownList {
          alignment:["fill", "fill"],
          minimumSize:["fill", #{MAXIMUM_SIZE}],
          helpTip:"Layer Type"
        },
      },

      cGrp2: Group {
        orientation:"row",
        spacing:0,
        alignment:["fill", "top"],
        margins:[0,0,0,0],

        colorBtn: Button {
          text: "",
          alignment:["fill", "fill"],
          minimumSize:[25, #{MAXIMUM_SIZE}],
          maximumSize:[5000, #{MAXIMUM_SIZE}],
          helpTip:"Solid Color"
        },

        trimBtn: Checkbox {
          text: "Trim",
          alignment:["fill", "fill"],
          minimumSize:[25, #{MAXIMUM_SIZE}],
          maximumSize:[5000, #{MAXIMUM_SIZE}],
          helpTip:""
        },
      },

      funGrp: Group {
        orientation:"row",
        spacing:0,
        margins:[0,0,0,0],
        alignment:["fill", "top"],

        addBtn: Button{
          text:"Add",
          minimumSize:[25, #{MAXIMUM_SIZE}],
          maximumSize:[5000, #{MAXIMUM_SIZE}],
          alignment:["fill", "fill"],
          helpTip:""
         },

        a: StaticText{
          spacing:0,
          margins:[0,0,0,0],
          minimumSize:[1, #{MAXIMUM_SIZE}],
          maximumSize:[1, #{MAXIMUM_SIZE}],
          alignment:["fill", "fill"],
          helpTip:""
         },

        helpBtn: Button{
          text:"?",
          minimumSize:[25, #{MAXIMUM_SIZE}],
          maximumSize:[25, #{MAXIMUM_SIZE}],
          alignment:["right", "fill"],
          helpTip:""
        },
      },
    }"""


      # test: StaticText {
      #   text:"Layer Type:",
      #   minimumSize:["fill", #{MAXIMUM_SIZE}],
      #   alignment:["fill", "fill"],
      #   helpTip:""
      # },

  myUI.grp = myUI.add(thisUI)
  myUI.layout.layout true
  myUI.margins = [1, 1, 1, 1]
  myUI.spacing = 2
  if parseFloat(app.version) > 11
    myUI.spacing = 1

  myUI.layout.resize()
  myUI.onResizing = myUI.onResize = ->
    @layout.resize()

  SOLID_LAYER   = "Solid"
  ADJUST_LAYER  = "Adjustment"
  NULL_LAYER_2D = "Null"
  NULL_LAYER_3D = "3D Null"
  CAMERA_LAYER  = "Camera"
  LIGHT_LAYER   = "Light"
  TEXT_LAYER    = "Text"
  SHAPE_LAYER   = "Shape"

  myUI.grp.oGrp1.layerTypeDdl.add("item", SOLID_LAYER)
  myUI.grp.oGrp1.layerTypeDdl.add("item", ADJUST_LAYER)
  myUI.grp.oGrp1.layerTypeDdl.add("item", NULL_LAYER_2D)
  myUI.grp.oGrp1.layerTypeDdl.add("item", NULL_LAYER_3D)
  myUI.grp.oGrp1.layerTypeDdl.add("item", CAMERA_LAYER)
  myUI.grp.oGrp1.layerTypeDdl.add("item", LIGHT_LAYER)
  myUI.grp.oGrp1.layerTypeDdl.add("item", TEXT_LAYER)
  myUI.grp.oGrp1.layerTypeDdl.add("item", SHAPE_LAYER)
  myUI.grp.oGrp1.layerTypeDdl.selection = 0

  customDraw = ->
    @graphics.drawOSControl()
    @graphics.rectPath 0, 0, @size[0], @size[1]
    @graphics.fillPath @fillBrush
    # @graphics.drawString @text, @textPen, (@size[0] - @graphics.measureString(@text, @graphics.font, @size[0])[0]) / 2, 3, @graphics.font  if @text
    return

  clrBtn = myUI.grp.cGrp2.colorBtn

  clrBtn.fillBrush = clrBtn.graphics.newBrush(clrBtn.graphics.BrushType.SOLID_COLOR, [1, 1, 1, 1])
  clrBtn.onDraw    = customDraw

  clrBtn.onClick = ->
    thisColor = $.colorPicker()
    return 0  if thisColor is "" or Null

    r = thisColor >> 16
    g = (thisColor & 0x00ff00) >> 8
    b = thisColor & 0xff

    BACK_COLOR = [r / 255, g / 255, b / 255]

    clrBtn.fillBrush = clrBtn.graphics.newBrush(clrBtn.graphics.BrushType.SOLID_COLOR, BACK_COLOR)

    clrBtn.onDraw = customDraw
    clrBtn.enabled = false
    clrBtn.enabled = true
    # dlg.layout.layout()
    return 0

  entryFunc = () ->
    actComp = app.project.activeItem
    return 0 unless isCompActive actComp

    selLayers = actComp.selectedLayers

    unless isLayerSelected selLayers
      switch String(myUI.grp.oGrp1.layerTypeDdl.selection)
        when SOLID_LAYER
          addedLayer = actComp.layers.addSolid(BACK_COLOR, "#{String(myUI.grp.oGrp1.layerTypeDdl.selection)}", actComp.width, actComp.height, actComp.pixelAspect, actComp.duration)
          break

        when ADJUST_LAYER
          addedLayer = actComp.layers.addSolid(BACK_COLOR, "#{String(myUI.grp.oGrp1.layerTypeDdl.selection)}", actComp.width, actComp.height, actComp.pixelAspect, actComp.duration)
          addedLayer.adjustmentLayer = true
          addedLayer.label = 5
          break

        when NULL_LAYER_2D
          addedLayer = actComp.layers.addNull(actComp.duration)
          addedLayer.name = "#{String(myUI.grp.oGrp1.layerTypeDdl.selection)}"
          break

        when NULL_LAYER_3D
          addedLayer = actComp.layers.addNull(actComp.duration)
          addedLayer.threeDLayer = true
          addedLayer.name = "#{String(myUI.grp.oGrp1.layerTypeDdl.selection)}"
          break

        when LIGHT_LAYER
          addedLayer = actComp.layers.addLight("#{String(myUI.grp.oGrp1.layerTypeDdl.selection)}", [actComp.width, actComp.height])
          break

        when CAMERA_LAYER
          addedLayer = actComp.layers.addCamera("#{String(myUI.grp.oGrp1.layerTypeDdl.selection)}", [actComp.width, actComp.height])
          break

        when TEXT_LAYER
          addedLayer = actComp.layers.addText("#{String(myUI.grp.oGrp1.layerTypeDdl.selection)}")
          break

        when SHAPE_LAYER
          addedLayer = actComp.layers.addShape()
          addedLayer.name = "#{String(myUI.grp.oGrp1.layerTypeDdl.selection)}"
          break

        else
      return 0


    allLayers = actComp.layers
    switch String(myUI.grp.oGrp1.layerTypeDdl.selection)
      when SOLID_LAYER
        for i in [0...selLayers.length] by 1
          selLayers[i].target = true

          for j in [1..allLayers.length] by 1
            if allLayers[j].target?
              addedLayer = actComp.layers.addSolid(BACK_COLOR, "#{String(myUI.grp.oGrp1.layerTypeDdl.selection)} #{i+1}", actComp.width, actComp.height, actComp.pixelAspect, actComp.duration)
              addedLayer.moveBefore(selLayers[i])

              if myUI.grp.cGrp2.trimBtn.value
                addedLayer.matchLength(selLayers[i])
              break

      when ADJUST_LAYER
        for i in [0...selLayers.length] by 1
          selLayers[i].target = true

          for j in [1..allLayers.length] by 1
            if allLayers[j].target?
              addedLayer = actComp.layers.addSolid(BACK_COLOR, "#{String(myUI.grp.oGrp1.layerTypeDdl.selection)} #{i+1}", actComp.width, actComp.height, actComp.pixelAspect, actComp.duration)
              addedLayer.adjustmentLayer = true
              addedLayer.label = 5
              addedLayer.moveBefore(selLayers[i])

              if myUI.grp.cGrp2.trimBtn.value
                addedLayer.matchLength(selLayers[i])
              break

      when NULL_LAYER_2D
        for i in [0...selLayers.length] by 1
          selLayers[i].target = true

          for j in [1..allLayers.length] by 1
            if allLayers[j].target?
              if myUI.grp.cGrp2.trimBtn.value
                addedLayer = actComp.layers.addNull(selLayers[i].outPoint - selLayers[i].inPoint)
              else
                addedLayer = actComp.layers.addNull(actComp.duration)

              addedLayer.name = "#{String(myUI.grp.oGrp1.layerTypeDdl.selection)} #{i+1}"
              addedLayer.moveBefore(selLayers[i])

              if myUI.grp.cGrp2.trimBtn.value
                addedLayer.matchLength(selLayers[i])
              break

      when NULL_LAYER_3D
        for i in [0...selLayers.length] by 1
          selLayers[i].target = true

          for j in [1..allLayers.length] by 1
            if allLayers[j].target?
              if myUI.grp.cGrp2.trimBtn.value
                addedLayer = actComp.layers.addNull(selLayers[i].outPoint - selLayers[i].inPoint)
              else
                addedLayer = actComp.layers.addNull(actComp.duration)

              addedLayer.name = "#{String(myUI.grp.oGrp1.layerTypeDdl.selection)} #{i+1}"
              addedLayer.threeDLayer = true
              addedLayer.moveBefore(selLayers[i])

              if myUI.grp.cGrp2.trimBtn.value
                addedLayer.matchLength(selLayers[i])
              break

      when LIGHT_LAYER
        for i in [0...selLayers.length] by 1
          selLayers[i].target = true

          for j in [1..allLayers.length] by 1
            if allLayers[j].target?
              addedLayer = actComp.layers.addLight("#{String(myUI.grp.oGrp1.layerTypeDdl.selection)} #{i+1}", [actComp.width, actComp.height])
              addedLayer.moveBefore(selLayers[i])

              if myUI.grp.cGrp2.trimBtn.value
                addedLayer.matchLength(selLayers[i])
              break

      when CAMERA_LAYER
        for i in [0...selLayers.length] by 1
          selLayers[i].target = true

          for j in [1..allLayers.length] by 1
            if allLayers[j].target?
              addedLayer = actComp.layers.addCamera("#{String(myUI.grp.oGrp1.layerTypeDdl.selection)} #{i+1}", [actComp.width, actComp.height])
              addedLayer.moveBefore(selLayers[i])

              if myUI.grp.cGrp2.trimBtn.value
                addedLayer.matchLength(selLayers[i])
              break

      when TEXT_LAYER
        for i in [0...selLayers.length] by 1
          selLayers[i].target = true

          for j in [1..allLayers.length] by 1
            if allLayers[j].target?
              addedLayer = actComp.layers.addText("#{String(myUI.grp.oGrp1.layerTypeDdl.selection)} #{i+1}")
              addedLayer.name = "#{String(myUI.grp.oGrp1.layerTypeDdl.selection)} #{i+1}"
              addedLayer.moveBefore(selLayers[i])

              if myUI.grp.cGrp2.trimBtn.value
                addedLayer.matchLength(selLayers[i])
              break

      when SHAPE_LAYER
        for i in [0...selLayers.length] by 1
          selLayers[i].target = true

          for j in [1..allLayers.length] by 1
            if allLayers[j].target?
              addedLayer = actComp.layers.addShape()
              addedLayer.name = "#{String(myUI.grp.oGrp1.layerTypeDdl.selection)} #{i+1}"
              addedLayer.moveBefore(selLayers[i])

              if myUI.grp.cGrp2.trimBtn.value
                addedLayer.matchLength(selLayers[i])
              break

      else

    return 0

  undoEntryFunc = (data) ->
    app.beginUndoGroup data.getScriptName()
    entryFunc()
    app.endUndoGroup()
    return 0


  myUI.grp.oGrp1.layerTypeDdl.onChange = () ->
    @selection = 0 unless @selection?
    return 0


  myUI.grp.funGrp.addBtn.onClick = ->
    undoEntryFunc SALData
    return 0

  myUI.grp.funGrp.helpBtn.onClick = ->
    AUTHOR_TEXT = "#{getLocalizedText({jp: "\n#{_STRINGS_JP.AUTHOR}\n\n#{_STRINGS_JP.HELP}",en: "\n#{_STRINGS_EN.AUTHOR}\n\n#{_STRINGS_EN.HELP}"})}"
    alert(
      SALData.getScriptName() + " " + SALData.getScriptVersionNumber() + AUTHOR_TEXT,
      SALData.getScriptName()
    )
    return 0

  return myUI


###
メイン処理開始
###
return 0 unless runAEVersionCheck SALData.getCanRunVersion()

msPanel = buildUI(@)

WHITE = [1, 1, 1]
BLACK = [0, 0, 0]
BACK_COLOR = WHITE

actComp = app.project.activeItem
if isCompActive actComp
  BACK_COLOR = app.project.activeItem.bgColor

unless msPanel instanceof Panel
  msPanel.center()
  msPanel.show()

return 0
