// Generated by CoffeeScript 1.12.4

/*
  Simple Add Layer
    (C) あかつきみさき(みくちぃP)

  このスクリプトについて
    コンポジション設定に基づいた新規レイヤーを作成します.
    レイヤーを選択している場合はその一つ上に作成します.

    追加可能な種類のレイヤー
      ・平面レイヤー
      ・調整レイヤー
      ・ヌルレイヤー
      ・ヌルレイヤー(3D)
      ・ライトレイヤー
      ・カメラレイヤー
      ・テキストレイヤー
      ・シェイプレイヤー

    Trim機能について
      新規レイヤーののスタートタイム・インポイント・アウトポイント・伸縮を,選択したレイヤーの値に一致させます.

  使用方法
    コンポジションをアクティブにしてからスクリプトを実行してください.

  動作環境
    Adobe After Effects CS5以上

  バージョン情報
    2018/10/11 Ver 1.1.0 Update
      調整レイヤー追加時,ラベルを通常追加時と同じ色にした.

    2014/08/07 Ver 1.0.0 Release
 */

(function() {
  var BACK_COLOR, BLACK, ColorClass, SALData, WHITE, _STRINGS_EN, _STRINGS_JP, actComp, buildUI, getLocalizedText, isCompActive, isLayerSelected, msPanel, runAEVersionCheck;

  SALData = (function() {
    var canRunVersion, guid, scriptName, scriptURLName, scriptURLVersion, scriptVersionNumber;
    scriptName = "Simple Add Layer";
    scriptURLName = "SimpleAddLayer";
    scriptVersionNumber = "1.1.0";
    scriptURLVersion = 1.10;
    canRunVersion = 10.0;
    guid = "{544E36ED-D16A-485F-B887-8B273C79291E}";
    return {
      getScriptName: function() {
        return scriptName;
      },
      getScriptURLName: function() {
        return scriptURLName;
      },
      getScriptVersionNumber: function() {
        return scriptVersionNumber;
      },
      getScriptURLVersion: function() {
        return scriptURLVersion;
      },
      getGuid: function() {
        return guid;
      },
      getCanRunVersion: function() {
        return canRunVersion;
      }
    };
  })();

  _STRINGS_JP = {
    AUTHOR: "あかつきみさき (SUNRISE MOON)",
    HELP: ""
  };

  _STRINGS_EN = {
    AUTHOR: "Misaki_Akatsuki (SUNRISE MOON)",
    HELP: ""
  };

  ColorClass = (function() {
    function ColorClass(name, colorType) {
      this.name = String(name);
      this.colorType = String(colorType);
      this.colorCode = name.split("#")[1];
      this.colorCode16 = String("0x" + this.colorCode);
      this.r = this.colorCode.slice(0, 2);
      this.g = this.colorCode.slice(2, 4);
      this.b = this.colorCode.slice(4, 6);
      this.rgb256 = [parseInt(this.r, 16), parseInt(this.g, 16), parseInt(this.b, 16), 1];
      this.rgb = [parseInt(this.r, 16) / 255, parseInt(this.g, 16) / 255, parseInt(this.b, 16) / 255, 1];
      this.hsv = rgb2hsv(this.rgb, true);
      this.h = this.hsv[0];
      this.s = this.hsv[1];
      this.v = this.hsv[2];
      this.cmyk = rgb2cmyk(this.rgb256);
      this.c = this.cmyk[0];
      this.m = this.cmyk[1];
      this.y = this.cmyk[2];
      this.k = this.cmyk[3];
    }

    return ColorClass;

  })();

  Settings.prototype.initSetting = function(sectionName, keyName, value) {
    if (!app.settings.haveSetting(sectionName, keyName)) {
      app.settings.saveSetting(sectionName, keyName, value);
    }
    return app.settings.getSetting(sectionName, keyName);
  };

  AVLayer.prototype.addSolidSmart = function(name) {
    this.containingComp.layers.addSolid(BLACK, "" + name, this.containingComp.width, this.containingComp.height, this.containingComp.pixelAspect, this.containingComp.duration);
    return this;
  };

  LayerCollection.prototype.addAdjustment = function(color, name, width, height, pixelAspect, duration) {
    var addLayer;
    addLayer = this.addSolid(color, name, width, height, pixelAspect, duration);
    addLayer.adjustmentLayer = true;
    return addLayer;
  };

  AVLayer.prototype.addAdjustmentSmart = function(name) {
    this.containingComp.layers.addAdjustment(BLACK, "" + name, this.containingComp.width, this.containingComp.height, this.containingComp.pixelAspect, this.containingComp.duration);
    return this;
  };

  TextLayer.prototype.matchLength = LightLayer.prototype.matchLength = ShapeLayer.prototype.matchLength = CameraLayer.prototype.matchLength = AVLayer.prototype.matchLength = function(layer) {
    if (!(layer instanceof AVLayer || layer instanceof LightLayer || layer instanceof CameraLayer || layer instanceof ShapeLayer || layer instanceof TextLayer)) {
      return false;
    }
    this.startTime = layer.startTime;
    this.stretch = layer.stretch;
    this.inPoint = layer.inPoint;
    this.outPoint = layer.outPoint;
    return true;
  };

  runAEVersionCheck = function(AEVersion) {
    if (parseFloat(app.version) < AEVersion) {
      alert("This script requires After Effects " + (AEVersion.toFixed(1)) + " or greater.");
      return false;
    } else {
      return true;
    }
  };

  isCompActive = function(selComp) {
    if (!(selComp && selComp instanceof CompItem)) {
      return false;
    } else {
      return true;
    }
  };

  isLayerSelected = function(selLayers) {
    if (selLayers.length === 0) {
      return false;
    } else {
      return true;
    }
  };

  getLocalizedText = function(str) {
    if (app.language === Language.JAPANESE) {
      return str.jp;
    } else {
      return str.en;
    }
  };

  buildUI = function(thisObj) {
    var ADJUST_LAYER, CAMERA_LAYER, LIGHT_LAYER, MAXIMUM_SIZE, NULL_LAYER_2D, NULL_LAYER_3D, SHAPE_LAYER, SOLID_LAYER, TEXT_LAYER, clrBtn, customDraw, entryFunc, myUI, thisUI, undoEntryFunc;
    MAXIMUM_SIZE = 20;
    myUI = (thisObj instanceof Panel ? thisObj : new Window("palette", SALData.getScriptName(), undefined, {
      resizeable: true
    }));
    thisUI = "Group {\n  alignment:[\"fill\", \"fill\"],\n  alignChildren:[\"fill\", \"top\"],\n  orientation:\"column\",\n  spacing: 1,\n  margins: [0, 0, 0, 0],\n\n  oGrp1: Group {\n    orientation:\"row\",\n    spacing:0,\n    margins:[0,0,0,0],\n    alignment:[\"fill\", \"top\"],\n\n    layerTypeDdl: DropDownList {\n      alignment:[\"fill\", \"fill\"],\n      minimumSize:[\"fill\", " + MAXIMUM_SIZE + "],\n      helpTip:\"Layer Type\"\n    },\n  },\n\n  cGrp2: Group {\n    orientation:\"row\",\n    spacing:0,\n    alignment:[\"fill\", \"top\"],\n    margins:[0,0,0,0],\n\n    colorBtn: Button {\n      text: \"\",\n      alignment:[\"fill\", \"fill\"],\n      minimumSize:[25, " + MAXIMUM_SIZE + "],\n      maximumSize:[5000, " + MAXIMUM_SIZE + "],\n      helpTip:\"Solid Color\"\n    },\n\n    trimBtn: Checkbox {\n      text: \"Trim\",\n      alignment:[\"fill\", \"fill\"],\n      minimumSize:[25, " + MAXIMUM_SIZE + "],\n      maximumSize:[5000, " + MAXIMUM_SIZE + "],\n      helpTip:\"\"\n    },\n  },\n\n  funGrp: Group {\n    orientation:\"row\",\n    spacing:0,\n    margins:[0,0,0,0],\n    alignment:[\"fill\", \"top\"],\n\n    addBtn: Button{\n      text:\"Add\",\n      minimumSize:[25, " + MAXIMUM_SIZE + "],\n      maximumSize:[5000, " + MAXIMUM_SIZE + "],\n      alignment:[\"fill\", \"fill\"],\n      helpTip:\"\"\n     },\n\n    a: StaticText{\n      spacing:0,\n      margins:[0,0,0,0],\n      minimumSize:[1, " + MAXIMUM_SIZE + "],\n      maximumSize:[1, " + MAXIMUM_SIZE + "],\n      alignment:[\"fill\", \"fill\"],\n      helpTip:\"\"\n     },\n\n    helpBtn: Button{\n      text:\"?\",\n      minimumSize:[25, " + MAXIMUM_SIZE + "],\n      maximumSize:[25, " + MAXIMUM_SIZE + "],\n      alignment:[\"right\", \"fill\"],\n      helpTip:\"\"\n    },\n  },\n}";
    myUI.grp = myUI.add(thisUI);
    myUI.layout.layout(true);
    myUI.margins = [1, 1, 1, 1];
    myUI.spacing = 2;
    if (parseFloat(app.version) > 11) {
      myUI.spacing = 1;
    }
    myUI.layout.resize();
    myUI.onResizing = myUI.onResize = function() {
      return this.layout.resize();
    };
    SOLID_LAYER = "Solid";
    ADJUST_LAYER = "Adjustment";
    NULL_LAYER_2D = "Null";
    NULL_LAYER_3D = "3D Null";
    CAMERA_LAYER = "Camera";
    LIGHT_LAYER = "Light";
    TEXT_LAYER = "Text";
    SHAPE_LAYER = "Shape";
    myUI.grp.oGrp1.layerTypeDdl.add("item", SOLID_LAYER);
    myUI.grp.oGrp1.layerTypeDdl.add("item", ADJUST_LAYER);
    myUI.grp.oGrp1.layerTypeDdl.add("item", NULL_LAYER_2D);
    myUI.grp.oGrp1.layerTypeDdl.add("item", NULL_LAYER_3D);
    myUI.grp.oGrp1.layerTypeDdl.add("item", CAMERA_LAYER);
    myUI.grp.oGrp1.layerTypeDdl.add("item", LIGHT_LAYER);
    myUI.grp.oGrp1.layerTypeDdl.add("item", TEXT_LAYER);
    myUI.grp.oGrp1.layerTypeDdl.add("item", SHAPE_LAYER);
    myUI.grp.oGrp1.layerTypeDdl.selection = 0;
    customDraw = function() {
      this.graphics.drawOSControl();
      this.graphics.rectPath(0, 0, this.size[0], this.size[1]);
      this.graphics.fillPath(this.fillBrush);
    };
    clrBtn = myUI.grp.cGrp2.colorBtn;
    clrBtn.fillBrush = clrBtn.graphics.newBrush(clrBtn.graphics.BrushType.SOLID_COLOR, [1, 1, 1, 1]);
    clrBtn.onDraw = customDraw;
    clrBtn.onClick = function() {
      var BACK_COLOR, b, g, r, thisColor;
      thisColor = $.colorPicker();
      if (thisColor === "" || Null) {
        return 0;
      }
      r = thisColor >> 16;
      g = (thisColor & 0x00ff00) >> 8;
      b = thisColor & 0xff;
      BACK_COLOR = [r / 255, g / 255, b / 255];
      clrBtn.fillBrush = clrBtn.graphics.newBrush(clrBtn.graphics.BrushType.SOLID_COLOR, BACK_COLOR);
      clrBtn.onDraw = customDraw;
      clrBtn.enabled = false;
      clrBtn.enabled = true;
      return 0;
    };
    entryFunc = function() {
      var actComp, addedLayer, allLayers, i, i1, j, k, l, m, n, o, p, q, ref, ref1, ref10, ref11, ref12, ref13, ref14, ref15, ref2, ref3, ref4, ref5, ref6, ref7, ref8, ref9, s, selLayers, t, u, v, w, x, y, z;
      actComp = app.project.activeItem;
      if (!isCompActive(actComp)) {
        return 0;
      }
      selLayers = actComp.selectedLayers;
      if (!isLayerSelected(selLayers)) {
        switch (String(myUI.grp.oGrp1.layerTypeDdl.selection)) {
          case SOLID_LAYER:
            addedLayer = actComp.layers.addSolid(BACK_COLOR, "" + (String(myUI.grp.oGrp1.layerTypeDdl.selection)), actComp.width, actComp.height, actComp.pixelAspect, actComp.duration);
            break;
          case ADJUST_LAYER:
            addedLayer = actComp.layers.addSolid(BACK_COLOR, "" + (String(myUI.grp.oGrp1.layerTypeDdl.selection)), actComp.width, actComp.height, actComp.pixelAspect, actComp.duration);
            addedLayer.adjustmentLayer = true;
            addedLayer.label = 5;
            break;
          case NULL_LAYER_2D:
            addedLayer = actComp.layers.addNull(actComp.duration);
            addedLayer.name = "" + (String(myUI.grp.oGrp1.layerTypeDdl.selection));
            break;
          case NULL_LAYER_3D:
            addedLayer = actComp.layers.addNull(actComp.duration);
            addedLayer.threeDLayer = true;
            addedLayer.name = "" + (String(myUI.grp.oGrp1.layerTypeDdl.selection));
            break;
          case LIGHT_LAYER:
            addedLayer = actComp.layers.addLight("" + (String(myUI.grp.oGrp1.layerTypeDdl.selection)), [actComp.width, actComp.height]);
            break;
          case CAMERA_LAYER:
            addedLayer = actComp.layers.addCamera("" + (String(myUI.grp.oGrp1.layerTypeDdl.selection)), [actComp.width, actComp.height]);
            break;
          case TEXT_LAYER:
            addedLayer = actComp.layers.addText("" + (String(myUI.grp.oGrp1.layerTypeDdl.selection)));
            break;
          case SHAPE_LAYER:
            addedLayer = actComp.layers.addShape();
            addedLayer.name = "" + (String(myUI.grp.oGrp1.layerTypeDdl.selection));
            break;
        }
        return 0;
      }
      allLayers = actComp.layers;
      switch (String(myUI.grp.oGrp1.layerTypeDdl.selection)) {
        case SOLID_LAYER:
          for (i = k = 0, ref = selLayers.length; k < ref; i = k += 1) {
            selLayers[i].target = true;
            for (j = l = 1, ref1 = allLayers.length; l <= ref1; j = l += 1) {
              if (allLayers[j].target != null) {
                addedLayer = actComp.layers.addSolid(BACK_COLOR, (String(myUI.grp.oGrp1.layerTypeDdl.selection)) + " " + (i + 1), actComp.width, actComp.height, actComp.pixelAspect, actComp.duration);
                addedLayer.moveBefore(selLayers[i]);
                if (myUI.grp.cGrp2.trimBtn.value) {
                  addedLayer.matchLength(selLayers[i]);
                }
                break;
              }
            }
          }
          break;
        case ADJUST_LAYER:
          for (i = m = 0, ref2 = selLayers.length; m < ref2; i = m += 1) {
            selLayers[i].target = true;
            for (j = n = 1, ref3 = allLayers.length; n <= ref3; j = n += 1) {
              if (allLayers[j].target != null) {
                addedLayer = actComp.layers.addSolid(BACK_COLOR, (String(myUI.grp.oGrp1.layerTypeDdl.selection)) + " " + (i + 1), actComp.width, actComp.height, actComp.pixelAspect, actComp.duration);
                addedLayer.adjustmentLayer = true;
                addedLayer.label = 5;
                addedLayer.moveBefore(selLayers[i]);
                if (myUI.grp.cGrp2.trimBtn.value) {
                  addedLayer.matchLength(selLayers[i]);
                }
                break;
              }
            }
          }
          break;
        case NULL_LAYER_2D:
          for (i = o = 0, ref4 = selLayers.length; o < ref4; i = o += 1) {
            selLayers[i].target = true;
            for (j = p = 1, ref5 = allLayers.length; p <= ref5; j = p += 1) {
              if (allLayers[j].target != null) {
                if (myUI.grp.cGrp2.trimBtn.value) {
                  addedLayer = actComp.layers.addNull(selLayers[i].outPoint - selLayers[i].inPoint);
                } else {
                  addedLayer = actComp.layers.addNull(actComp.duration);
                }
                addedLayer.name = (String(myUI.grp.oGrp1.layerTypeDdl.selection)) + " " + (i + 1);
                addedLayer.moveBefore(selLayers[i]);
                if (myUI.grp.cGrp2.trimBtn.value) {
                  addedLayer.matchLength(selLayers[i]);
                }
                break;
              }
            }
          }
          break;
        case NULL_LAYER_3D:
          for (i = q = 0, ref6 = selLayers.length; q < ref6; i = q += 1) {
            selLayers[i].target = true;
            for (j = s = 1, ref7 = allLayers.length; s <= ref7; j = s += 1) {
              if (allLayers[j].target != null) {
                if (myUI.grp.cGrp2.trimBtn.value) {
                  addedLayer = actComp.layers.addNull(selLayers[i].outPoint - selLayers[i].inPoint);
                } else {
                  addedLayer = actComp.layers.addNull(actComp.duration);
                }
                addedLayer.name = (String(myUI.grp.oGrp1.layerTypeDdl.selection)) + " " + (i + 1);
                addedLayer.threeDLayer = true;
                addedLayer.moveBefore(selLayers[i]);
                if (myUI.grp.cGrp2.trimBtn.value) {
                  addedLayer.matchLength(selLayers[i]);
                }
                break;
              }
            }
          }
          break;
        case LIGHT_LAYER:
          for (i = t = 0, ref8 = selLayers.length; t < ref8; i = t += 1) {
            selLayers[i].target = true;
            for (j = u = 1, ref9 = allLayers.length; u <= ref9; j = u += 1) {
              if (allLayers[j].target != null) {
                addedLayer = actComp.layers.addLight((String(myUI.grp.oGrp1.layerTypeDdl.selection)) + " " + (i + 1), [actComp.width, actComp.height]);
                addedLayer.moveBefore(selLayers[i]);
                if (myUI.grp.cGrp2.trimBtn.value) {
                  addedLayer.matchLength(selLayers[i]);
                }
                break;
              }
            }
          }
          break;
        case CAMERA_LAYER:
          for (i = v = 0, ref10 = selLayers.length; v < ref10; i = v += 1) {
            selLayers[i].target = true;
            for (j = w = 1, ref11 = allLayers.length; w <= ref11; j = w += 1) {
              if (allLayers[j].target != null) {
                addedLayer = actComp.layers.addCamera((String(myUI.grp.oGrp1.layerTypeDdl.selection)) + " " + (i + 1), [actComp.width, actComp.height]);
                addedLayer.moveBefore(selLayers[i]);
                if (myUI.grp.cGrp2.trimBtn.value) {
                  addedLayer.matchLength(selLayers[i]);
                }
                break;
              }
            }
          }
          break;
        case TEXT_LAYER:
          for (i = x = 0, ref12 = selLayers.length; x < ref12; i = x += 1) {
            selLayers[i].target = true;
            for (j = y = 1, ref13 = allLayers.length; y <= ref13; j = y += 1) {
              if (allLayers[j].target != null) {
                addedLayer = actComp.layers.addText((String(myUI.grp.oGrp1.layerTypeDdl.selection)) + " " + (i + 1));
                addedLayer.name = (String(myUI.grp.oGrp1.layerTypeDdl.selection)) + " " + (i + 1);
                addedLayer.moveBefore(selLayers[i]);
                if (myUI.grp.cGrp2.trimBtn.value) {
                  addedLayer.matchLength(selLayers[i]);
                }
                break;
              }
            }
          }
          break;
        case SHAPE_LAYER:
          for (i = z = 0, ref14 = selLayers.length; z < ref14; i = z += 1) {
            selLayers[i].target = true;
            for (j = i1 = 1, ref15 = allLayers.length; i1 <= ref15; j = i1 += 1) {
              if (allLayers[j].target != null) {
                addedLayer = actComp.layers.addShape();
                addedLayer.name = (String(myUI.grp.oGrp1.layerTypeDdl.selection)) + " " + (i + 1);
                addedLayer.moveBefore(selLayers[i]);
                if (myUI.grp.cGrp2.trimBtn.value) {
                  addedLayer.matchLength(selLayers[i]);
                }
                break;
              }
            }
          }
          break;
      }
      return 0;
    };
    undoEntryFunc = function(data) {
      app.beginUndoGroup(data.getScriptName());
      entryFunc();
      app.endUndoGroup();
      return 0;
    };
    myUI.grp.oGrp1.layerTypeDdl.onChange = function() {
      if (this.selection == null) {
        this.selection = 0;
      }
      return 0;
    };
    myUI.grp.funGrp.addBtn.onClick = function() {
      undoEntryFunc(SALData);
      return 0;
    };
    myUI.grp.funGrp.helpBtn.onClick = function() {
      var AUTHOR_TEXT;
      AUTHOR_TEXT = "" + (getLocalizedText({
        jp: "\n" + _STRINGS_JP.AUTHOR + "\n\n" + _STRINGS_JP.HELP,
        en: "\n" + _STRINGS_EN.AUTHOR + "\n\n" + _STRINGS_EN.HELP
      }));
      alert(SALData.getScriptName() + " " + SALData.getScriptVersionNumber() + AUTHOR_TEXT, SALData.getScriptName());
      return 0;
    };
    return myUI;
  };


  /*
  メイン処理開始
   */

  if (!runAEVersionCheck(SALData.getCanRunVersion())) {
    return 0;
  }

  msPanel = buildUI(this);

  WHITE = [1, 1, 1];

  BLACK = [0, 0, 0];

  BACK_COLOR = WHITE;

  actComp = app.project.activeItem;

  if (isCompActive(actComp)) {
    BACK_COLOR = app.project.activeItem.bgColor;
  }

  if (!(msPanel instanceof Panel)) {
    msPanel.center();
    msPanel.show();
  }

  return 0;

}).call(this);
